import MockResponse from "./MockResponse";
import MockRequest from "./MockRequest";
import MockProgressEvent from "./polyfill/MockProgressEvent";
import MockEvent from "./polyfill/MockEvent";
import MockXMLHttpRequestUpload from "./MockXMLHttpRequestUpload";
const notImplementedError = new Error("This feature hasn't been implmented yet. Please submit an Issue or Pull Request on Github.");
const createEvent = (options, target, type) => {
    const hasProgress = ["error", "progress", "loadstart", "loadend", "load",
        "timeout", "abort"];
    let event;
    if (hasProgress.indexOf(type) > -1) {
        event = new MockProgressEvent(type, {
            lengthComputable: true,
            loaded: options && options.loaded || 0,
            total: options && options.total || 0,
        });
    }
    else {
        event = new MockEvent(type);
    }
    event.currentTarget = target;
    event.target = target;
    return event;
};
export default class MockXMLHttpRequest {
    constructor() {
        // Somehow they are both defined on the instance and static method
        this.UNSENT = 0;
        this.OPENED = 1;
        this.HEADERS_RECEIVED = 2;
        this.LOADING = 3;
        this.DONE = 4;
        // some libraries (like Mixpanel) use the presence of this field to check
        // if XHR is properly supported. See
        // https://developer.mozilla.org/en-US/docs/Web/API/XMLHttpRequest/withCredentials
        this.withCredentials = false;
        this.timeout = 0;
        this._events = [];
        this.reset();
        this.upload = new MockXMLHttpRequestUpload();
    }
    /** Add a request handler */
    static addHandler(fn) {
        MockXMLHttpRequest.handlers.push(fn);
    }
    /** Remove a request handler */
    static removeHandler(fn) {
        throw notImplementedError;
    }
    /** Remove all request handlers */
    static reset() {
        this.handlers = [];
    }
    /** Handle a request */
    static handle(req) {
        for (const handler of MockXMLHttpRequest.handlers) {
            // get the generator to create a response to the request
            const response = handler(req, new MockResponse());
            if (response) {
                return response;
            }
        }
        return null;
    }
    get handlers() {
        return MockXMLHttpRequest.handlers;
    }
    dispatchEvent(evt) {
        throw notImplementedError;
    }
    /** Reset the response values */
    reset() {
        this._requestHeaders = {};
        this._responseHeaders = {};
        this.status = 200;
        this.statusText = "";
        this.response = null;
        this.responseType = null;
        this.responseText = null;
        this.responseXML = null;
        this.readyState = MockXMLHttpRequest.UNSENT;
    }
    /** Trigger an event */
    trigger(type, options) {
        if (this.onreadystatechange) {
            this.onreadystatechange.call(this, createEvent(options, this, "readystatechange"));
        }
        const hasEvent = this._events.find(item => item.type === "loadend");
        if (this.readyState === MockXMLHttpRequest.DONE &&
            (this.onloadend || hasEvent)) {
            let listener;
            if (this.onloadend) {
                listener = this.onloadend;
            }
            else if (typeof hasEvent !== "undefined") {
                listener = hasEvent.listener;
            }
            if (typeof listener !== "undefined") {
                listener.call(this, createEvent(options, this, "loadend"));
            }
        }
        if (this["on" + type]) {
            this["on" + type].call(this, createEvent(options, this, type));
        }
        for (const event of this._events) {
            if (event.type === type) {
                event.listener.call(this, createEvent(options, this, type));
            }
        }
        return this;
    }
    open(method, url, async, user, password) {
        this.reset();
        this.readyState = MockXMLHttpRequest.OPENED;
        this.data = null;
        if (typeof url === "undefined") {
            this.url = method;
        }
        else {
            this.method = method;
            this.url = url;
            this.async = async;
            this.user = user;
            this.password = password;
        }
    }
    setRequestHeader(name, value) {
        this._requestHeaders[name] = value;
    }
    overrideMimeType(mime) {
        throw notImplementedError;
    }
    send(data) {
        this.readyState = MockXMLHttpRequest.LOADING;
        this.data = data;
        setTimeout(() => {
            const response = MockXMLHttpRequest.handle(new MockRequest(this));
            if (response !== null) {
                const timeout = Math.max(response.timeout(), this.timeout);
                if (timeout > 0) {
                    // trigger a timeout event because the request timed timeout
                    // - wait for the timeout time because many libs like jquery
                    // and superagent use setTimeout to detect the error type
                    this._sendTimeout = setTimeout(() => {
                        this.readyState = MockXMLHttpRequest.DONE;
                        this.trigger("timeout");
                    }, timeout);
                }
                else {
                    // map the response to the XHR object
                    this.status = response.status();
                    this._responseHeaders = response.headers();
                    this.responseType = "text";
                    this.response = response.body();
                    this.responseText = response.body(); // TODO: detect an object and return JSON, detect XML and return XML
                    this.readyState = MockXMLHttpRequest.DONE;
                    // trigger a load event because the request was received
                    this.trigger("loadstart");
                    this.trigger("load");
                }
            }
            else {
                // trigger an error because the request was not handled
                this.readyState = MockXMLHttpRequest.DONE;
                this.trigger("error");
            }
        }, 0);
    }
    abort() {
        clearTimeout(this._sendTimeout);
        if (this.readyState > MockXMLHttpRequest.UNSENT &&
            this.readyState < MockXMLHttpRequest.DONE) {
            this.readyState = MockXMLHttpRequest.UNSENT;
            this.trigger("abort");
        }
    }
    getAllResponseHeaders() {
        if (this.readyState < MockXMLHttpRequest.HEADERS_RECEIVED) {
            return null;
        }
        // Yes the spec states \r\n line-breaks...
        return Object.keys(this._responseHeaders)
            .map(key => key + ": " + this._responseHeaders[key] + "\r\n")
            .join("");
    }
    getResponseHeader(name) {
        if (this.readyState < MockXMLHttpRequest.HEADERS_RECEIVED) {
            return null;
        }
        return this._responseHeaders[name.toLowerCase()] || null;
    }
    addEventListener(type, listener, useCapture) {
        this._events.push({
            listener,
            type,
        });
    }
    removeEventListener(type, listener) {
        let i = 0;
        while (i < this._events.length) {
            const item = this._events[i];
            if (item.type === type && item.listener === listener) {
                this._events.splice(i, 1);
            }
            else {
                i++;
            }
        }
    }
}
MockXMLHttpRequest.UNSENT = 0;
MockXMLHttpRequest.OPENED = 1;
MockXMLHttpRequest.HEADERS_RECEIVED = 2;
MockXMLHttpRequest.LOADING = 3;
MockXMLHttpRequest.DONE = 4;
MockXMLHttpRequest.handlers = [];
//# sourceMappingURL=MockXMLHttpRequest.js.map